%% =========================================================================
% prefix1: the input dir name
% path0: Reference frame. It usually is LR frames
% path1: Input frames: It can be any super-resolved HR frames. E.g. It can
% be super-resolved frames using ScSR or NLM or Texture-synthesis-based SR.
% path2: Output dir
% path3: SR mean frame. It may improve the performance. Texture-synthesis
% SR needs mean image over time-axes during synthesis. If we provide the
% super-resolved mean image, the performance could be improved. (It's
% optional)

% patchSize: The size of patch to be synthesized. Note that it cannot be
% too smaller because the motion basis need more information to be
% estimated.
% scale factor: The enlarge factor.
%% =========================================================================

function dynaSynt(path0, path1, path2, path3, path4, para)

% path0 = ['.\TestSequence\' prefix1 '\Small\'];     
% path1 = ['.\TestSequence\' prefix1 '\TSS_BOBMC\']; 
% path2 = ['.\TestSequence\' prefix1 '\TSS_dynaSR\']; 
% path3 = ['.\TestSequence\' prefix1 '\meanSmall\'];  
% path4 = ['.\TestSequence\' prefix1 '\Original\'];

patchSize=para.patchSize;
scaleFactor=para.scaleFactor;
overlap=para.overlap;
len=para.len;

if (~exist(path2, 'dir')), mkdir(path2); end
dx = dir([path0 '*.jpg']);
dxx = dir([path1 '*.jpg']);

frameNum = 1:len;
Step = 55;
myn = 2;
overlap2 = 0;
method =0;

%% Preprocessing

bimg = imread([path1 dxx(1).name]);
yimg = bimg(:,:,1);
Zy = im2col_overlap(yimg, [patchSize patchSize], overlap);
block_num = size(Zy, 2);
[nrow, ncol, iscolor] = size(bimg);

I = [];
sI = [];

BB = zeros(patchSize^2, block_num, 1, len);
SS = zeros(patchSize^2, block_num, 1, len);
TT = zeros(patchSize^2, block_num, 1, len);

%% Collect the temporal information from videos

for o=1:len
    simg = imread([path0 dx(o).name]);
    bimg = imread([path1 dxx(o).name]);
    
    simg = imresize(simg, size(bimg(:,:,1)));

    timg = imresize(imread([path4 dx(o).name]), size(simg(:,:,1)));
    
    ycbcrimg = double(rgb2ycbcr(uint8(bimg)));
    ycbcrimg2 = double(rgb2ycbcr(uint8(simg)));
    ycbcrimg3 = double(rgb2ycbcr(uint8(timg)));
    for k=1:3
        Zy = im2col_overlap(ycbcrimg(:,:,k), [patchSize patchSize], overlap);
        sZy = im2col_overlap(ycbcrimg2(:,:,k), [patchSize patchSize], overlap);
        tZy = im2col_overlap(ycbcrimg3(:,:,k), [patchSize patchSize], overlap);
        
        BB(:,:,k,o) = Zy;
        SS(:,:,k,o) = sZy;
        TT(:,:,k,o) = tZy;
      
    end
    
    clc;
    disp(['Total Progress is ' num2str(o/len * 100) '%' ]);
end
save BBSS4.mat BB SS TT;
load BBSS4;


%% Step: The synthesized dynamics may result in overflow, therefore the dynamic should be learnt step by step.

seq1 = [];
for bk = 1:block_num
    
    %     load BBSS;
    for p=1:Step:len-Step
        
        for c=1:3
            I = [];  sI = [];  tI = []; ssrI = [];
            for o=p:p+Step;
                I = [I BB(:,bk,c,o)];
                sI = [sI SS(:,bk,c,o)];
                tI = [tI TT(:,bk,c,o)];
            end
            
            if (std(std(double(I)))>0.15)
                [x0,Ymean,Ahat,Bhat,Chat] = dytex_pair(double(sI), double(I), double(tI), 56, 2, method, BB(:,bk,c,1));
                I2 = synth(x0,Ymean,Ahat,Bhat,Chat,len);                
                i = 1;
                for o=p:p+Step
                    BB(:,bk,c,o) = I2(:, i);
                    i = i + 1;
                end
                
            else
                break;
            end
        end
        
    end
    clc;
    disp(['Total Progress is ' num2str(bk/block_num * 100) '%' ]);
end
plot(seq1);

%% Reading the segmentaton map. Only dynamic regions will be synthesized.
segF = false;
try
    segPath = para.segPath;
    seg1 = imread(segPath);
    seg1 = repmat(im2bw(imresize(seg1, size(bimg(:,:,1)))), [1 1 3]);
    segF = true;
catch
    segF = false;
end

for o=1:len
    bimg = imread([path1 dxx(o).name]);
    bimg = im2double(bimg)*255;
    
    ycbcrimg = rgb2ycbcr(uint8(bimg));
    for c=1:3
        ycbcrimg(:,:,c) = col2im_overlap(BB(:,:,c,o), [patchSize patchSize], [nrow ncol], overlap,[],0);
    end
    bimg=ycbcr2rgb(uint8(ycbcrimg));
    
     cc = imresize(imread([path0 dxx(o).name]), size(bimg(:,:,1)));
     m1 = bimg(:,:,1)==0 & bimg(:,:,2)==136 & bimg(:,:,3)==0;
     m1 = repmat(m1, [1 1 3]);
     bimg(m1==1)=cc(m1==1);
        
    if (segF)
        bimg(seg1==0)=cc(seg1==0);
    end
    progressbar(o, len);
    imwrite(bimg,[path2 dxx(o).name]);
end
disp('Done! Congratulations~!');
%%
